import pygame
from Loader import Loader
from pygame.locals import *
import math
import random

class Particle:
	def __init__(self,surface,position,velocity,alpha=255):
		self.surface = pygame.Surface(surface.get_size())
		self.original_surface = pygame.Surface.convert(surface)
		self.overlay = pygame.Surface(self.surface.get_size())
		self.overlay.fill((0,0,0))
		self.position = position
		self.velocity = velocity
		self.alpha = alpha
		self.alive = True
		self.size = self.surface.get_size()
	
	def update(self):
		old_alpha = self.alpha
		self.alpha -= 4
		if self.alpha > old_alpha or self.alpha <= 0:
			self.alive = 0
			self.alpha = 0
			return False
		self.surface.fill( (0,0,0) )
		self.surface.blit( self.original_surface, (0,0),None,BLEND_ADD )
		self.surface.blit( self.original_surface, (0,0),None,BLEND_ADD )
		self.surface.blit( self.original_surface, (0,0),None,BLEND_ADD )
		self.overlay.set_alpha(255-self.alpha)
		self.surface.blit( self.overlay, (0,0) )
		self.position[0] += self.velocity[0]
		self.position[1] += self.velocity[1]
		return True
	
	def draw(self,screen):
		if self.alpha > 0:
			screen.blit(self.surface,(self.position[0]-self.size[0]/2,self.position[1]-self.size[1]/2),None,BLEND_ADD)

class Source:
	def __init__(self,position,color,count):
		self.position = position
		self.color = color
		self.count = count
		particle_surface = Loader().load_art("particle.png")[0]
		self.surface = pygame.Surface.convert(pygame.Surface(particle_surface.get_size()))
		self.surface.fill(color)
		self.surface.blit(particle_surface,(0,0),None,BLEND_RGB_MULT)
		self.particles = []
		for i in range(0,count):
			size = random.random()*24
			particle = Particle(pygame.transform.scale(self.surface,(size,size)),[position[0]+random.random()*10-5,position[1]+random.random()*10-5],(random.random()*10-5,random.random()*10-5),random.random()*255)
			self.particles.append(particle)
	
	def update(self):
		alive = False
		dead = []
		for particle in self.particles:
			if particle.update():
				alive = True
			else:
				dead.append(particle)
		for particle in dead:
			self.particles.remove(particle)
		return alive
	
	def draw(self,screen):
		for particle in self.particles:
			particle.draw(screen)
		return True