import pygame
from pygame.locals import *
from Loader import Loader

class Button:
	def __init__(self,string,pos,size=16):
		self.pos = pos
		self.string = None
		self.tsize = size
		self.bg_color = (64,64,64)
		self.hg_color = (128,128,128)
		self.write(string)
		self.message = string
		self.enabled = True
	
	def disable(self,alse=True):
		if alse:
			self.bg_color = (196,196,196)
			self.enabled = False
		else:
			self.bg_color = (64,64,64)
			self.enabled = True
		pygame.draw.rect(self.surface,self.bg_color,pygame.Rect(2,2,self.size[0]-4,self.size[1]-4),0)
		self.surface.blit(self.text,(10,7))
	
	def toggle(self):
		if self.enabled == False:
			return False
		if self.bg_color[0] == 64:
			self.bg_color = (255,64,64)
			self.hg_color = (255,128,128)
		else:
			self.bg_color = (64,64,64)
			self.hg_color = (128,128,128)
		if not self.highlight:
			pygame.draw.rect(self.surface,self.bg_color,pygame.Rect(2,2,self.size[0]-4,self.size[1]-4),0)
		else:
			pygame.draw.rect(self.surface,self.hg_color,pygame.Rect(2,2,self.size[0]-4,self.size[1]-4),0)
		self.surface.blit(self.text,(10,7))
		return True
	
	def write(self,string,text_color = (255,255,255),border_color = (196,196,196)):
		if string == self.string:
			return False

		load = Loader()
		font = load.load_font("slkscr",self.tsize)
		text = font.render(string,False,text_color)
		self.text = text
		self.surface = pygame.Surface((text.get_size()[0]+20,text.get_size()[1]+14))
		self.surface.fill(self.bg_color)
		self.surface.blit(text,(10,7))
		pygame.draw.rect(self.surface,border_color,pygame.Rect(0,0,self.surface.get_size()[0]-1,self.surface.get_size()[1]-1),2)
		self.size = self.surface.get_size()
		self.string = string
		self.highlight = False
		return True
	
	def click(self,pos):
		return self.enabled and self.mouse_in(pos)
	
	def draw(self,surface):
		surface.blit(self.surface,self.pos)
		return True
	
	def update(self):
		return True
	
	def mouse_in(self,coords):
		if self.enabled == False:
			return False
		if coords[0] > self.pos[0] and coords[0] < self.pos[0] + self.size[0] and coords[1] > self.pos[1] and coords[1] < self.pos[1] + self.size[1]:
			if not self.highlight:
				pygame.draw.rect(self.surface,self.hg_color,pygame.Rect(2,2,self.size[0]-4,self.size[1]-4),0)
				self.surface.blit(self.text,(10,7))
				self.highlight = True
			return True
		if self.highlight:
			pygame.draw.rect(self.surface,self.bg_color,pygame.Rect(2,2,self.size[0]-4,self.size[1]-4),0)
			self.surface.blit(self.text,(10,7))
			self.highlight = False
		return False
