# An example course
#resolution=1280x800
#wall 10 10 0 780
import os
import sys
import time
import pygame
import httplib
import urllib
from State import State
from Wall import Wall
from Ball import Ball
from Label import Label
from Loader import Loader

from constants import *

def to_str(val): return str(val)

class Course:
  def __init__(self,filename,physics,folder="courses"):
    self.objects = []
    self.statics = []
    self.cue = False
    self.physics = physics
    self.text = []
    self.resolution = State().screen_size
    self.font = Loader().load_font("slkscr",16)
    self.par = 2
    self.author = State().username
    self.legal = False
    self.best = False
    self.filename = "temp"
    self.created = int(time.time())
    self.offset = (0,0)
    self.id = 0
    self.rating = 0
    
    if filename != False:
      fin = open(os.path.join(folder,filename+".course"),"r")
      self.filename = filename
      for line in fin:
        self.process_line(line)
      fin.close()
  
  def process_line(self,line):
    line = line.strip()
    if line.find("=") != -1: # option
      temp = line.split("=")
      key = temp[0]
      value = temp[1].strip()
      if key == "resolution":
        res = value[1:-1].split(", ")
        self.resolution = (int(res[0]),int(res[1]))
        if self.resolution[0] <= State().screen_size[0] and self.resolution[1] <= State().screen_size[1]:
          self.offset = (State().screen_size[0]/2 - self.resolution[0]/2,State().screen_size[1]/2 - self.resolution[1]/2)
          self.legal = True
      elif key == "par":
        self.par = int(value)
      elif key == "author":
        self.author = value
      elif key == "best":
        self.best = int(value)
      elif key == "created":
        self.created = int(value)
      elif key == "id":
        self.id = int(value)
      elif key == "rating":
        self.rating = int(value)
    elif line.lstrip().startswith("#"):
      True # ignore comments
    else:
      opts = line.strip().split(" ")
      if self.physics != None: # element
        self.add(opts)
      else:
        self.text.append( map(to_str,opts) )
  
  def reset(self):
    text = self.text
    self.clear()
    for opt in text:
      self.add(opt)
  
  def save(self,filename=False):
    if not filename:
      filename = self.filename
    if self.id != 0:
      filename = "download_" + str(self.id)
    if len(self.text) < 1:
      return False
    fout = open(os.path.join("courses",filename + ".course"),"w")
    fout.write("resolution="+str(self.resolution)+"\n")
    fout.write("author="+str(self.author)+"\n")
    fout.write("par="+str(self.par)+"\n")
    fout.write("created="+str(self.created)+"\n")
    fout.write("id="+str(self.id)+"\n")
    fout.write("rating="+str(self.rating)+"\n")
    if self.best != False:
      fout.write("best="+str(self.best)+"\n")
    for line in self.text:
      fout.write( " ".join(line) + "\n" )
    fout.close()
  
  def add(self,opts):
    object = None
    self.text.append( map(to_str,opts) )
    if opts[0] == "wall":
      object = Wall(self.physics.space,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),(0,0),(int(opts[3]),int(opts[4])),WALL)
      self.statics.append(object)
    elif opts[0] == "cue":
      self.cue = Ball(self.physics,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),15,10,CUE)
      self.objects.append(self.cue)
      object = self.cue
    elif opts[0] == "glass":
      object = Ball(self.physics,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),23,10,GLASS)
      self.objects.append( object )
    elif opts[0] == "wood":
      object = Ball(self.physics,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),32,2,WOOD)
      self.objects.append( object )
    elif opts[0] == "rubber":
      object = Ball(self.physics,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),7,0.01,RUBBER)
      self.objects.append( object )
    elif opts[0] == "target":
      object = Ball(self.physics,(int(opts[1])+self.offset[0],int(opts[2])+self.offset[1]),31,1e100,TARGET)
      self.objects.append( object )
    if object:
      object.label = opts[0]
    return object
  
  def remove_object(self,obj):
    self.physics.space.remove(obj.shape)
    self.objects.remove(obj)
    
  def upload(self):
    try:
      server = State().server
      url = "/api/upload_course.php"
      course_string = ""
      for line in self.text:
        course_string += " ".join(line)+"\n"
      course_string = course_string.strip()
      params = urllib.urlencode({'course':course_string, 'author':self.author, 'resolution':str(self.resolution), 'par':str(self.par)})
      headers = {"Content-type": "application/x-www-form-urlencoded","Accept": "text/plain"}
      connection = httplib.HTTPConnection(server,80)
      connection.request("POST", url, params, headers)
      response = connection.getresponse()
      if response.status == 200 and response.reason == "OK":
        self.id = int(response.read())
        return True
    except:
      return False
    finally:
      connection.close()
    return False
  
  def rate(self,rating):
    try:
      server = State().server
      url = "/api/do_rating.php"
      params = urllib.urlencode({'id':self.id, 'rating':rating, 'username':State().username})
      headers = {"Content-type": "application/x-www-form-urlencoded","Accept": "text/plain"}
      connection = httplib.HTTPConnection(server,80)
      connection.request("POST", url, params, headers)
      response = connection.getresponse()
      print url,params,response.status,response.reason
      if response.status == 200 and response.reason == "OK":
        print response.read()
        return True
    except:
      return False
    finally:
      connection.close()
    return False
  
  def clear(self):
    for obj in self.statics:
      self.physics.space.remove_static(obj.shape)
    self.statics = []
    for obj in self.objects:
      self.physics.space.remove(obj.shape)
    self.objects = []
    self.text = []
    
  def update(self):
    for obj in self.objects:
      obj.update()
    return True
  
  def draw(self,screen):
    for obj in self.statics:
      obj.draw(screen)
    for obj in self.objects:
      obj.draw(screen)
    return True