import pygame
from Loader import Loader
from State import State
from Widget import Widget

class StaticText(Widget):
	def __init__(self,x,y,string,color=(196,196,196),size=8):
		self.font_size = size
		self.color = color
		self.shadow_s = None
		self.string = None
		self.x = x
		self.y = y
		self.set_text(string)
	
	def render(self):
		self.surface = Loader().load_font(State().font_name,self.font_size).render(self.string.replace("_"," "),True,self.color)
		size = self.surface.get_size()
		Widget.__init__(self,self.x,self.y,size[0],size[1])
	
	def set_text(self,str):
		if self.string != str:
			self.string = str
			self.render()
		
	def get_text(self):
		return self.string
			
	def shadow(self,bool=True,depth=5):
		if bool:
			shadow_color = (self.color[0]/depth,self.color[1]/depth,self.color[2]/depth)
			self.shadow_s = Loader().load_font(State().font_name,self.font_size).render(self.string.replace("_"," "),False,shadow_color)
		else:
			self.shadow_s = None
	
	def align(self,dir="center",width=None):
		if width == None:
			width = State().screen_size[0]
		if dir == "center":
			self.x += width/2-self.size[0]/2
		elif dir == "left":
			self.x = 5
		elif dir == "right":
			self.x = width-self.size[0]-5
	
	def set_alpha(self,alpha):
		Widget.set_alpha(self,alpha)
		self.surface.set_alpha(self.alpha)
	
	def draw(self,surf):
		if self.visible:
			if self.shadow_s:
				surf.blit(self.shadow_s,(self.x+1,self.y+1))
			surf.blit(self.surface,(self.x,self.y))
	
	def mouseover(self,x,y):
		return False

class Paragraph(Widget):
	def __init__(self,x,y,string,color=(196,196,196),size=16):
		self.labels = []
		indent = 10
		_y = y
		w = 0
		h = 0
		for str in string.split("\n"):
			txt = StaticText(x+indent,_y,str,color,size)
			self.labels.append(txt)
			_y += txt.size[1]+4
			indent = 0
			h += txt.height
			if txt.width > w:
				w = txt.width
		Widget.__init__(self,x,y,w,h)
	
	def align(self,dir,width=None):
		for lbl in self.labels:
			lbl.align(dir,width)
	
	def draw(self,surf):
		for lbl in self.labels:
			lbl.draw(surf)
	
	def mouseover(self,x,y):
		return False