from AbstractScreen import AbstractScreen
import os
import sys
import time
import pygame
import util
from pymunk import vec2d
from Loader import Loader
from State import State
from Label import Label
from SoundManager import SoundManager
from Button import Button
from pygame.locals import *
from PhysicsManager import PhysicsManager
from Course import Course

class EditorScreen(AbstractScreen):
	# Constructor
	def __init__(self):
		AbstractScreen.__init__(self)
		State().open_editor_with = False
		self.shift = False
		self.physics = PhysicsManager()
		self.active_button = False
		self.tool = "Move"
		self.course = Course(False,self.physics)
		self.wall_start = False
		self.cue = False
		self.target = False
		self.message = False
		self.buttons = []
		self.buttons.append( Button("Clear",(10,10),16) )
		self.buttons.append( Button("Wall",(self.buttons[0].pos[0] + self.buttons[0].size[0] + 40,10),16) )
		self.buttons.append( Button("Cue",(self.buttons[1].pos[0] + self.buttons[1].size[0] + 10,10),16) )
		self.buttons.append( Button("Target",(self.buttons[2].pos[0] + self.buttons[2].size[0] + 10,10),16) )
		self.buttons.append( Button("Glass",(self.buttons[3].pos[0] + self.buttons[3].size[0] + 40,10),16) )
		self.buttons.append( Button("Wood",(self.buttons[4].pos[0] + self.buttons[4].size[0] + 10,10),16) )
		self.buttons.append( Button("Rubber",(self.buttons[5].pos[0] + self.buttons[5].size[0] + 10,10),16) )
		self.buttons.append( Button("Save",(10 + self.buttons[6].size[0] + 40,self.state.screen_size[1]-self.buttons[5].size[1]-10),16) )
		self.buttons.append( Button("Upload",(self.buttons[7].pos[0] + self.buttons[7].size[0] + 10,self.state.screen_size[1]-self.buttons[5].size[1]-10),16) )
		self.buttons.append( Button("Quit Editor",(self.buttons[8].pos[0] + self.buttons[8].size[0] + 10,self.state.screen_size[1]-self.buttons[5].size[1]-10),16) )
		self.buttons.append( Button("Delete",(self.buttons[6].pos[0] + self.buttons[6].size[0] + 20,10),16) )
		#self.buttons.append( Button("Quit Editor",(self.buttons[9].pos[0] + self.buttons[9].size[0] + 40,self.state.screen_size[1]-self.buttons[5].size[1]-10),16) )
		
		self.buttons[7].disable()
		self.buttons[8].disable()
		
		for button in self.buttons:
			self.attach(button)
	
	# Call this when the screen is switched to. Can we make this automatic?
	def on_focus(self):
		if self.state.username == "anonymous":
			self.state.game.switch_to(7)
			return True
		self.target = False
		self.cue = False
		if self.state.open_editor_with:
			self.course = Course(self.state.open_editor_with,self.physics)
			self.buttons[7].disable(False)
			self.buttons[8].disable(False)
		else:
			self.course = Course(False,self.physics)
			self.buttons[7].disable()
			self.buttons[8].disable()
		return True
	
	# Draw this screen, returning True on success
	def draw(self,screen):
		screen.fill( (0,0,0) )
		AbstractScreen.draw(self,screen)
		self.course.draw(screen)
		return True
	
	def handle_event(self,event):
		if event.type == KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				return False
			elif event.key == 304: # Shift
				self.shift = True
		elif event.type == KEYUP:
			if event.key == 304: # Shift
				self.shift = False
		elif event.type == MOUSEBUTTONUP:
			return True # does nothing
		elif event.type == MOUSEMOTION:
			for button in self.buttons:
				button.mouse_in(event.pos)
		elif event.type == MOUSEBUTTONDOWN:
			if self.message != False:
				self.stage.remove(self.message)
				self.message = False
				return True
			for button in self.buttons:
				if button.click(event.pos):
					if button.message == "Wall" or button.message == "Cue" or button.message == "Glass" or button.message == "Wood" or button.message == "Rubber" or button.message == "Target" or button.message == "Delete":
						button.toggle()
						if self.active_button == False:
							self.active_button = button
							self.tool = button.message
						elif button == self.active_button:
							self.active_button = False
							self.tool = "Move"
						else:
							self.active_button.toggle()
							self.active_button = button
							self.tool = button.message
						return True
					elif button.message == "Clear":
						self.course.clear()
						self.cue = False
						self.target = False
						self.buttons[7].disable()
						self.buttons[8].disable()
						State().open_editor_with = False
						return True
					elif button.message == "Play":
						return True
					elif button.message == "Save":
						self.course.author = self.state.username
						filename = State().username + "_" + str(int(time.time()))
						if self.state.open_editor_with != False:
							filename = self.state.open_editor_with
						self.course.save(filename)
						return True
					elif button.message == "Upload":
						self.message = Label("Uploading...",Loader().load_font("slkscr",48),(255,255,255),(0,0))
						self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
						self.message.alpha = 255
						self.message.set_shadow(True)
						self.stage.attach(self.message)
						if self.course.upload():
							self.stage.remove(self.message)
							self.message = Label("Upload Successful",Loader().load_font("slkscr",48),(196,255,196),(0,0))
							self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
							self.stage.attach(self.message)
						else:
							self.stage.remove(self.message)
							self.message = Label("Upload Failed",Loader().load_font("slkscr",48),(255,196,196),(0,0))
							self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
							self.stage.attach(self.message)
						return True
					elif button.message == "Quit Editor":
						State().open_editor_with = False
						self.state.game.switch_to(1)
						return True
					return True # Stop if we handled a button click
			# If we got here we must not have clicked a button...
			if self.tool == "Wall":
				if self.wall_start == False:
					self.wall_start = event.pos
				else:
					offset = [event.pos[0] - self.wall_start[0],event.pos[1] - self.wall_start[1]]
					if self.shift != False:
						if abs(event.pos[0] - self.wall_start[0]) < abs(event.pos[1] - self.wall_start[1]):
							offset[0] = 0
						else:
							offset[1] = 0
					self.course.add(["wall",self.wall_start[0],self.wall_start[1],offset[0],offset[1]])
					self.wall_start = False
			elif self.tool == "Cue":
				if self.cue != False:
					self.course.remove_object(self.cue)
					self.course.text.remove( [self.cue.label,str(int(self.cue.body.position[0])),str(int(self.cue.body.position[1]))] )
				self.cue = self.course.add(["cue",event.pos[0],event.pos[1]])
			elif self.tool == "Glass":
				self.course.add(["glass",event.pos[0],event.pos[1]])
			elif self.tool == "Wood":
				self.course.add(["wood",event.pos[0],event.pos[1]])
			elif self.tool == "Rubber":
				self.course.add(["rubber",event.pos[0],event.pos[1]])
			elif self.tool == "Target":
				print self.target
				if self.target != False:
					self.course.remove_object(self.target)
					self.course.text.remove( [self.target.label,str(int(self.target.body.position[0])),str(int(self.target.body.position[1]))] )
				self.target = self.course.add(["target",event.pos[0],event.pos[1]])
			else:
				remove = None
				for ball in self.course.objects:
					if util.distance(ball.body.position,vec2d.Vec2d(event.pos)) < ball.radius+3:
						remove = ball
						break
				if remove:
					print remove.label
					if remove.label == "target" or remove.label == "cue":
						self.buttons[7].disable()
						self.buttons[8].disable()
					self.course.objects.remove(remove)
					self.course.text.remove( [remove.label,str(int(remove.body.position[0])),str(int(remove.body.position[1]))] )
			if self.target and self.cue:
				self.buttons[7].disable(False)
				self.buttons[8].disable(False)
		return True
	
	# Return value:
	#	True -- successful update
	#	False -- Error in update OR done, and switch to previous screen
	def update(self):
		self.course.update()
		return AbstractScreen.update(self)