import pygame
import httplib
import urllib
import os
import time
from PhysicsManager import PhysicsManager
from Course import Course
from State import State
from SurfaceButton import SurfaceButton
from AbstractScreen import AbstractScreen
from Label import Label
from Loader import Loader
from Button import Button
from pygame.locals import *
from Sprite import Sprite

class GalleryScreen(AbstractScreen):
	def __init__(self):
		AbstractScreen.__init__(self)
		self.state = State()
		self.title_font = Loader().load_font("slkscr",96)
		self.small_font = Loader().load_font("slkscr",16)
		title = Label("Courses", self.title_font, (255,255,255), (0,0))
		title.set_x(self.state.screen_size[0]/2 - title.size[0]/2)
		title.set_shadow(True)
		self.attach( title )
		self.message = False
		self.previous = Button("<",(5,0),32)
		self.previous.pos = (self.previous.pos[0],self.state.screen_size[1]/2 - self.previous.size[1]/2)
		self.attach(self.previous)
		self.next = Button(">",(5,0),32)
		self.next.pos = (self.state.screen_size[0] - self.next.size[0]-5,self.state.screen_size[1]/2 - self.next.size[1]/2)
		self.attach(self.next)
		self.back = Button("Back to Menu",(5,self.state.screen_size[1]-64),32)
		self.back.pos = (10,self.state.screen_size[1] - self.back.size[1] - 10)
		self.attach(self.back)
		self.download = Button("Download More Courses",(5,self.state.screen_size[1]-64),32)
		self.download.pos = (self.state.screen_size[0] - self.download.size[0] - 10,self.state.screen_size[1] - self.download.size[1] - 10)
		self.attach(self.download)
		
		# Get last download time
		fin = open(os.path.join("res","data","last_checked.txt"),"r")
		self.state.last_checked = int(fin.readline().strip())
		fin.close()
		
		self.thumbs = []
		self.edit_buttons = []
		self.delete_buttons = []
		self.index = 0
		self.on_focus()
		
	def render_thumbs(self):
		for button in self.edit_buttons:
			self.stage.remove(button)
		for button in self.delete_buttons:
			self.stage.remove(button)
		index = self.index
		self.thumbs = []
		x_space = (self.state.screen_size[0] - (self.state.screen_size[0]/4*3))/4
		y_space = x_space
		y_start = 128
		self.edit_buttons = []
		self.delete_buttons = []
		for i in range(index,index+6):
			if i >= len(self.courses):
				break
			physics = PhysicsManager()
			surface = pygame.Surface( self.state.screen_size )
			surface.fill( (0,0,0) )
			course = Course(self.courses[i], physics)
			course.draw(surface)
			course.clear()
			thumb = pygame.transform.smoothscale( surface, (self.state.screen_size[0]/4,self.state.screen_size[1]/4) )
			par = Label("Par " + str(course.par),self.small_font,(255,255,255),(5,5))
			par.set_shadow(True)
			par.alpha = 255
			par.draw(thumb)
			author = Label(course.author,self.small_font,(255,255,255),(5,5))
			author.set_y(self.state.screen_size[1]/4 - 5 - author.height)
			author.set_shadow(True)
			author.alpha = 255
			author.draw(thumb)
			if course.best:
				best_color = (255,64,64)
				if course.best  < course.par:
					best_color = (64,255,0)
				best = Label("Best: "+str(course.best),self.small_font,best_color,(5,5))
				best.set_x(self.state.screen_size[0]/4-5-best.width)
				best.set_shadow(True)
				best.alpha = 255
				best.draw(thumb)
			if course.rating > 0:
				for x in range(1,course.rating+1):
					star = Sprite(Loader().load_art("star_small.png")[0],(self.state.screen_size[0]/4-12*x-2,self.state.screen_size[1]/4-12))
					star.draw(thumb)
			j = i - index
			coords = (x_space + (j % 3) * (self.state.screen_size[0]/4 + x_space), y_start + (j / 3) * (self.state.screen_size[1]/4 + y_space))
			self.thumbs.append( (self.courses[i], SurfaceButton(thumb,coords)) )
			edit_button = Button("Edit",(coords[0]+self.state.screen_size[0]/8,coords[1]+self.state.screen_size[1]/4+2),16)
			edit_button.pos = (edit_button.pos[0]-edit_button.size[0]/2,edit_button.pos[1])
			self.edit_buttons.append(edit_button)
			self.attach(edit_button)
			delete_button = Button("Delety",(coords[0]+self.state.screen_size[0]/4,coords[1]+self.state.screen_size[1]/4+2),8)
			delete_button.pos = (delete_button.pos[0]-delete_button.size[0],delete_button.pos[1])
			delete_button.bg_color = (128,0,0)
			delete_button.hg_color = (128,64,64)
			delete_button.write("Delete",(255,196,196),(255,128,128))
			self.delete_buttons.append(delete_button)
			self.attach(delete_button)
		if index + 6 >= len(self.courses):
			self.next.disable()
		else:
			self.next.disable(False)
		if index <= 0:
			self.previous.disable()
		else:
			self.previous.disable(False)
	
	def download_courses(self):
		try:
			server = self.state.server
			url = "/api/download_courses.php" # TODO: /api/...  
			params = urllib.urlencode({'last_checked':str(self.state.last_checked)})
			headers = {"Content-type": "application/x-www-form-urlencoded","Accept": "text/plain"}
			connection = httplib.HTTPConnection(server,80)
			connection.request("POST", url, params, headers)
			response = connection.getresponse()
			print response.status, response.reason
			if response.status == 200:
				html = response.read()
				courses = html.split("//SPLIT//")
				for text in courses:
					course = Course(False,None)
					for line in text.split("\n"):
						course.process_line(line)
						course.save(course.author + "_" + str(course.created))
			else:
				return False
		except:
			return False
		finally:
			try:
				connection.close()
			except IOError:
				return False
		self.state.last_checked = int(time.time())
		fout = open(os.path.join("res","data","last_checked.txt"),"w")
		fout.write(str(self.state.last_checked)+"\n")
		fout.close()
		old_index = self.index
		self.on_focus()
		self.index = old_index
		self.render_thumbs()
		return True
		
	def on_focus(self):
		self.courses = []
		self.index = 0
		for file in os.listdir("courses"):
			if file.endswith("course"):
				name = file.split(".")[0]
				course = Course(name,None)
				if course.legal:
					self.courses.append(name)
		self.state.courses = self.courses
		self.render_thumbs()
		return True
	
	def draw(self,screen):
		self.screen_cache = screen # naughty...
		screen.fill( (0,0,0) )
		for thumb in self.thumbs:
			thumb[1].draw(screen)
		AbstractScreen.draw(self,screen)
		return True
	
	def update(self):
		AbstractScreen.update(self)
		return True

	def handle_event(self,event):
		if event.type == MOUSEBUTTONDOWN:
			if self.message != False:
				self.stage.remove(self.message)
				self.message = False
				return True
			for pair in self.thumbs:
				if pair[1].click(event.pos):
					self.state.course_to_play = pair[0]
					self.state.game.switch_to(3)
					return True
			if self.next.click(event.pos):
				self.index += 6
				self.render_thumbs()
				return True
			elif self.previous.click(event.pos):
				self.index -= 6
				self.render_thumbs()
				return True
			elif self.back.click(event.pos):
				self.state.game.switch_to(1)
				return True
			elif self.download.click(event.pos):
				self.message = Label("Downloading...",Loader().load_font("slkscr",48),(255,255,255),(0,0))
				self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
				self.message.alpha = 255
				self.message.set_shadow(True)
				self.stage.attach(self.message)
				self.message.draw(self.screen_cache)
				if self.download_courses():
					self.stage.remove(self.message)
					self.message = Label("Download Successful",Loader().load_font("slkscr",48),(196,255,196),(0,0))
					self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
					self.message.set_shadow(True)
					self.stage.attach(self.message)
				else:
					self.stage.remove(self.message)
					self.message = Label("Download Failed",Loader().load_font("slkscr",48),(255,196,196),(0,0))
					self.message.coords = (self.state.screen_size[0]/2-self.message.size[0]/2,self.state.screen_size[1]/2-self.message.size[1]/2)
					self.message.set_shadow(True)
					self.stage.attach(self.message)
				return True
			edit_index = self.index
			for button in self.edit_buttons:
				if button.mouse_in(event.pos):
					self.state.open_editor_with = self.courses[edit_index]
					self.state.game.switch_to(4)
				edit_index += 1
			delete_index = self.index
			for button in self.delete_buttons:
				if button.mouse_in(event.pos):
					os.remove(os.path.join("courses",self.courses[delete_index] + ".course"))
					old_index = self.index
					self.on_focus()
					self.index = old_index
					self.render_thumbs()
				delete_index += 1
		elif event.type == MOUSEMOTION:
			for button in [self.next,self.previous,self.back,self.download]:
				button.mouse_in(event.pos)
			for button in self.edit_buttons:
				button.mouse_in(event.pos)
			for button in self.delete_buttons:
				button.mouse_in(event.pos)
			for thumb in self.thumbs:
				thumb[1].mouse_in(event.pos)
		elif event.type == KEYDOWN:
			self.state.game.switch_to(1)
		return True