import pygame
from AbstractScreen import AbstractScreen
from Loader import Loader
from Sprite import Sprite
from PhysicsManager import PhysicsManager
from Course import Course
from pygame.locals import *
from Label import Label
from constants import *
from AdditiveParticle import Source
import random
import time
import math

class MenuScreen(AbstractScreen):
	def __init__(self):
		random.seed(time.time())
		AbstractScreen.__init__(self)
		self.physics = PhysicsManager(1.0)
		self.physics.setup_collisions()
		self.physics.set_callback("sparks",self.sparks)
		self.course = Course(False,self.physics)
		self.attach(self.course)
		self.on_focus()
		
		title = Sprite(Loader().load_art("title.png")[0],(0,0),BLEND_ADD)
		title.coords = (self.state.screen_size[0]/2 - title.size[0]/2,10)
		self.attach(title)
		item_font = Loader().load_font("slkscr",48)
		self.tutorial = Label("Tutorial",item_font,(255,255,255),(30,title.coords[1]+title.size[1]+60))
		self.tutorial.set_shadow(True)
		self.tutorial.goal_alpha = 215
		self.attach(self.tutorial)
		self.gallery = Label("Play Course",item_font,(255,255,255),(30,self.tutorial.coords[1]+self.tutorial.size[1]+30))
		self.gallery.set_shadow(True)
		self.gallery.goal_alpha = 196
		self.attach(self.gallery)
		self.editor = Label("Design New Course",item_font,(255,255,255),(30,self.gallery.coords[1]+self.gallery.size[1]+10))
		self.editor.set_shadow(True)
		self.editor.goal_alpha = 128
		self.attach(self.editor)
		self.options = Label("Options",item_font,(255,255,255),(30,self.editor.coords[1]+self.editor.size[1]+10))
		self.options.set_shadow(True)
		self.attach(self.options)
		self.quit = Label("Quit",item_font,(255,255,255),(30,self.options.coords[1]+self.options.size[1]+10))
		self.quit.set_shadow(True)
		self.quit.goal_alpha = 128
		self.attach(self.quit)
	
	def sparks(self,position,types):
		colors = [(255,255,255),(255,255,255),(64,255,64),(255,64,64),(255,255,64),(255,255,255),(64,128,255)]
		self.attach(Source(position,colors[types[0]],3))
		self.attach(Source(position,colors[types[1]],3))
		# Play glockenspiel
		if types[1] == GLASS:
			self.state.sound.play("glass")
		elif types[1] == WOOD:
			self.state.sound.play("wood")
		elif types[1] == RUBBER:
			self.state.sound.play("rubber")
		elif types[1] == TARGET:
			self.state.sound.play("target")
		elif types[1] == CUE:
			self.state.sound.play("cue")
		return True
	
	def on_focus(self):
		# Decoration
		self.course.clear()
		self.course.add( ["wall",-2,-2,-2,self.state.screen_size[1]+4] )
		self.course.add( ["wall",-2,-2,self.state.screen_size[0]+4,-2] )
		self.course.add( ["wall",-2,self.state.screen_size[1]+2,self.state.screen_size[0]+4,0] )
		self.course.add( ["wall",self.state.screen_size[0]+4,-2,0,self.state.screen_size[1]+4] )
		for i in range(0,5):
			ball = self.course.add( ["glass", random.random()*self.state.screen_size[0]-60 + 30, random.random()*self.state.screen_size[1]-60 + 30])
			angle = random.random()*(math.pi * 2)
			ball.body.apply_impulse( (math.cos(angle)*150,math.sin(angle)*150) )
		for i in range(0,2):
			ball = self.course.add( ["wood", random.random()*self.state.screen_size[0]-60 + 30, random.random()*self.state.screen_size[1]-60 + 30])
			angle = random.random()*(math.pi * 2)
			ball.body.apply_impulse( (math.cos(angle)*100,math.sin(angle)*100) )
		for i in range(0,3):
			ball = self.course.add( ["rubber", random.random()*self.state.screen_size[0]-60 + 30, random.random()*self.state.screen_size[1]-60 + 30])
			angle = random.random()*(math.pi * 2)
			ball.body.apply_impulse( (math.cos(angle)*400,math.sin(angle)*400) )
		cue= self.course.add( ["cue", random.random()*self.state.screen_size[0]-60 + 30, random.random()*self.state.screen_size[1]-60 + 30])
		angle = random.random()*(math.pi * 2)
		cue.body.apply_impulse( (math.cos(angle)*100,math.sin(angle)*100) )
		
		# Music
		track = self.state.sound.get_track_number("theme")
		self.state.sound.fade_music()
#		self.state.sound.start_music(track)
		return True
	
	def draw(self,screen):
		screen.fill( (0,0,0) )
		AbstractScreen.draw(self,screen)
		return True
	
	def update(self):
		self.physics.update()
		AbstractScreen.update(self)
		return True

	def handle_event(self,event):
		if event.type == MOUSEMOTION:
			for label in [self.tutorial, self.gallery, self.editor, self.options, self.quit]:
				if label.mouse_in(event.pos):
					label.goal_alpha = 255
				else:
					label.goal_alpha = 128
		elif event.type == MOUSEBUTTONDOWN:
			if self.gallery.mouse_in(event.pos):
				self.state.game.switch_to(2)
			elif self.tutorial.mouse_in(event.pos):
				self.state.game.switch_to(6)
			elif self.editor.mouse_in(event.pos):
				self.state.game.switch_to(4)
			elif self.options.mouse_in(event.pos):
				self.state.game.switch_to(5)
			elif self.quit.mouse_in(event.pos):
				return False
		return True