from AbstractScreen import AbstractScreen
import os
import sys
import pygame
from pygame.locals import *
from Loader import Loader
from State import State
from Label import Label
from SoundManager import SoundManager
from StaticText import StaticText

class OptionsScreen(AbstractScreen):
	def __init__(self):
		AbstractScreen.__init__(self)
		self.loader = Loader()
		self.state = State()
		self.state.font_name = "slkscr"
		x = self.state.screen_size[0]/4
		y = self.state.screen_size[1]/6
		
		self.labels =     [StaticText(0,y,"Resolution",(255,255,255),72)]
		self.labels[0].align("center")
		y += self.labels[0].size[1] + 30
		for str in ["800x600","1024x768","1024x600 (Netbook)", "1280x800 (Wide)","1280x1024","1680x1050 (Wide)"]:
			lbl = StaticText(x,y,str,(255,255,255),32)
			y += lbl.height+10
			self.labels.append(lbl)
		note = StaticText(x+20,y,"Press Enter to save or Escape to go back",(255,255,255),16)
		self.labels.append(note)
		y += note.height
		self.changes = StaticText(x,y,"Changes take effect on exit",(255,200,10),16)
		self.changes.align("center")
		self.changes.visible = False
		self.selected = 1
		for label in self.labels:
			label.shadow()
		self.full_lbl = [StaticText(x*2,self.state.screen_size[1]/6+self.labels[0].height+30,"Fullscreen OFF",(64,64,64),32),
										 StaticText(x*2,self.state.screen_size[1]/6+self.labels[0].height+30,"Fullscreen ON",(255,255,255),32)]
		self.full_sel = 0
		if self.state.fullscreen:
			self.full_sel = 1
		for lbl in self.full_lbl:
			lbl.shadow(True)
		self.column = 0
	
	def on_focus(self):
		self.alive = True
		
	def update(self):
		AbstractScreen.update(self)
		for label in self.labels:
			label.update()
		return True
	
	def handle_event(self,event):
		if event.type == pygame.locals.KEYDOWN:
			if event.key == K_ESCAPE:
				if self.changes.visible:
					os.execv(sys.executable,(sys.executable,sys.argv[0]))
				else:
					self.state.game.switch_to(1)
				return True
			elif event.key == K_UP and self.selected > 1:
				self.selected -= 1
			elif event.key == K_DOWN and self.selected < len(self.labels)-2:
				self.selected += 1
			elif event.key == K_LEFT or event.key == K_RIGHT:
				self.column += 1
				if self.column > 1:
					self.column = 0
					self.labels[len(self.labels)-1].visible = True
				else:
					self.labels[len(self.labels)-1].visible = False
			elif event.key == K_RETURN:
				if self.column == 0:
					resolution = (800,600)
					if self.selected == 1:
						resolution = (800,600)
					elif self.selected == 2:
						resolution = (1024,768)
					elif self.selected == 3:
						resolution = (1024,600)
					elif self.selected == 4:
						resolution = (1280,800)
					elif self.selected == 5:
						resolution = (1280,1024)
					elif self.selected == 6:
						resolution = (1680,1050)
					fout = open(os.path.join("res","data","resolution.txt"),"w")
					fout.write( str(resolution[0])+"x"+str(resolution[1]) )
					fout.close()
				else:
					self.full_sel += 1
					if self.full_sel > 1:
						self.full_sel = 0
					fout = open(os.path.join("res","data","fullscreen.txt"),"w")
					fout.write( str(self.full_sel) )
					fout.close()
				self.changes.visible = True
		return True
	
	def draw(self,screen):
		screen.fill( (0,0,0) )
		if self.column == 0:
			sel = self.labels[self.selected]
		else:
			sel = self.full_lbl[0]
		pygame.draw.rect(screen,(32,32,32),pygame.Rect(sel.coords[0]-10,sel.coords[1]-3,sel.width+20,sel.height+6))
		for label in self.labels:
			label.draw(screen)
		self.full_lbl[self.full_sel].draw(screen)
		self.changes.draw(screen)
		return True
