from AbstractScreen import AbstractScreen
import os
from sys import *
import pygame
from Loader import Loader
from State import State
from Label import Label
from Button import Button
from Sprite import Sprite
from SoundManager import SoundManager
from PhysicsManager import PhysicsManager
from pygame.locals import *
from constants import *
import util
import random
from pymunk import vec2d
from AdditiveParticle import Source
from Course import Course

class PlayScreen(AbstractScreen):
	# Constructor
	def __init__(self):
		AbstractScreen.__init__(self)
		self.physics = PhysicsManager()
		self.physics.setup_collisions()
		self.physics.set_callback("victory",self.on_win)
		self.physics.set_callback("sparks",self.sparks)
		self.state = State()
		self.course = False
		self.drag_start = False
		self.drag_lines = False
		self.stroke = 0
		self.font = Loader().load_font("slkscr",32)
		self.small_font = Loader().load_font("slkscr",16)
		self.par_label = False
		self.course_name = False
		self.in_overlay = False
		self.overlay = []
		fade = pygame.Surface(self.state.screen_size)
		fade.fill( (0,0,0) )
		fade.set_alpha(220)
		self.fade = Sprite(fade,(0,0))
		self.gallery = Button("Back to Gallery",(0,0),32)
		self.gallery.pos = (self.state.screen_size[0]/2 - self.gallery.size[0]/2,self.state.screen_size[1]/2)
		self.random = Button("Play Random Hole",(0,50),32)
		self.random.pos = (self.state.screen_size[0]/2 - self.random.size[0]/2,self.state.screen_size[1]/2 + self.gallery.size[1] + 5)
		self.stars = []
		x = self.state.screen_size[0]/2 - (92*5)/2
		for i in range(0,5):
			self.stars.append(Sprite(Loader().load_art("star_big.png")[0],(x,self.random.pos[1]+self.random.size[1]+10),BLEND_ADD,1))
			x += 92
		self.pity = False
		
	def sparks(self,position,types):
		colors = [(255,255,255),(255,255,255),(64,255,64),(255,64,64),(255,255,64),(255,255,255),(64,128,255)]
		self.attach(Source(position,colors[types[0]],5))
		self.attach(Source(position,colors[types[1]],5))
		# Play glockenspiel
		if types[1] == GLASS:
			self.state.sound.play("glass")
		elif types[1] == WOOD:
			self.state.sound.play("wood")
		elif types[1] == RUBBER:
			self.state.sound.play("rubber")
		elif types[1] == TARGET:
			self.state.sound.play("target")
		elif types[1] == CUE:
			self.state.sound.play("cue")
		return True
	
	def on_win(self):
		self.in_overlay = True
		self.overlay = []
		self.overlay.append( self.fade )
		
		score = "Par"
		color = (255,255,255)
		if self.stroke > self.course.par: # Over par
			score = str(self.stroke - self.course.par) + " over par"
			color = (255,128,128)
		elif self.stroke == self.course.par: # Par
			score = "Par"
		elif self.course.par - self.stroke == 1:
			score = "Birdie"
			color = (196,255,196)
		elif self.course.par - self.stroke == 2:
			score = "Eagle"
			color = (128,255,128)
		elif self.course.par - self.stroke == 3:
			score = "Albatross"
			color = (64,255,64)
		else:
			score = str(self.course.par - self.stroke) + " under"
			color = (0,255,0)
		score = Label(score,self.font,color,(0,0))
		score.coords = (self.state.screen_size[0]/2-score.size[0]/2,self.state.screen_size[1]/2 - 100)
		score.set_shadow(True)
		score.alpha = 255
		self.overlay.append( score )
		self.overlay.append( self.gallery )
		self.overlay.append( self.random )
		for star in self.stars:
			self.overlay.append( star )
		best = None
		if self.course.best == False or self.course.best > self.stroke:
			self.course.best = self.stroke
			self.course.save()
			best = Label("New record!",self.font,(64,255,64),(0,0))
		else:
			best = Label("Your record: "+str(self.course.best) + " strokes",self.font,(255,255,255),(0,0))
		best.coords = (self.state.screen_size[0]/2-best.size[0]/2,self.state.screen_size[1]/2 - 60)
		best.set_shadow(True)
		best.alpha = 255
		self.overlay.append(best)
	
	def reset(self):
		self.load_course(self.course_name)
	
	def load_course(self,coursename):
		if self.course != False:
			self.stage.remove(self.course)
			self.course.clear()
			self.stage.remove(self.author)
		if self.pity != False:
			self.stage.remove(self.pity)
			self.pity = False
		self.in_overlay = False
		self.stroke = 0
		self.course = Course(coursename,self.physics)
		self.attach(self.course)
		if self.par_label != False:
			self.stage.remove(self.par_label)
		self.par_label = Label("Stroke "+str(self.stroke) + " / Par " + str(self.course.par),self.font,(255,255,255),(20,20))
		self.par_label.goal_alpha = 196
		self.par_label.set_shadow(True)
		self.stage.attach(self.par_label)
		author = Label(self.course.author,self.small_font,(255,255,255),(0,0))
		author.set_x(self.state.screen_size[0] - author.size[0]-20)
		author.set_y(self.state.screen_size[1] - author.size[1]-20)
		author.goal_alpha = 196
		author.set_shadow(True)
		self.author = author
		self.stage.attach(author)
		self.course_name = coursename
	
	def on_focus(self):
		if self.state.course_to_play:
			self.load_course(self.state.course_to_play)
			self.state.sound.fade_music()
		return True
	
	# Draw this screen, returning True on success
	def draw(self,screen):
		screen.fill( (0,0,0) )
		if self.drag_start != False and self.drag_lines != False:
			for line in self.drag_lines:
				pygame.draw.aaline(screen,(255,255,255),line[0],line[1],4)
		AbstractScreen.draw(self,screen)
		if self.in_overlay:
			for obj in self.overlay:
				obj.draw(screen)
		return True

	def update_drag(self,end):
		distance = (end[0] - self.drag_start.x, end[1] - self.drag_start.y)
		self.drag_lines = []
		length = util.distance(self.drag_start,vec2d.Vec2d(end))
		seglen = 16
		sx = self.drag_start.x
		sy = self.drag_start.y
		dx = distance[0]/(length/seglen + 1)
		dy = distance[1]/(length/seglen + 1)
		for i in range(0,length/seglen):
			self.drag_lines.append( ((sx,sy),(sx+dx/2,sy+dy/2)) )
			sx += dx
			sy += dy
	
	def finish_drag(self,end):
		self.course.cue.body.apply_impulse((end[0] - self.drag_start.x, end[1] - self.drag_start.y),(0,0))
		self.drag_lines = False
		self.drag_start = False
		if self.par_label != False:
			self.stage.remove(self.par_label)
		self.stroke += 1
		self.par_label = Label("Stroke "+str(self.stroke) + " / Par " + str(self.course.par),self.font,(255,255,255),(20,20))
		self.par_label.goal_alpha = 196
		self.par_label.set_shadow(True)
		self.stage.attach(self.par_label)
		if self.stroke > 4:
			self.pity = Label("Press 'Space' to reset course, or 'Escape' to go give up.",self.small_font,(255,255,255),(0,0))
			self.pity.set_x(self.state.screen_size[0]/2-self.pity.size[0]/2)
			self.pity.set_y(self.state.screen_size[1]-self.pity.size[1]-5)
			self.stage.attach(self.pity)
	
	def update(self):
		if not self.in_overlay:
			self.physics.update()
		AbstractScreen.update(self)
		return True

	def handle_event(self,event):
		if event.type == KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				self.state.game.switch_to(2)
			elif event.key == 32: # Space
				self.reset()
		elif event.type == MOUSEBUTTONUP:
			if self.drag_start != False:
				self.finish_drag(event.pos)
		elif event.type == MOUSEMOTION:
			if self.in_overlay:
				for btn in [self.gallery,self.random]:
					btn.mouse_in(event.pos)
				star_index = 0
				for star in self.stars:
					star.render_count = 1
					if star.mouse_in(event.pos):
						for i in range(0,star_index+1):
							self.stars[i].render_count = 3
					star_index += 1
				return True
			if self.drag_start != False:
				self.update_drag(event.pos)
		elif event.type == MOUSEBUTTONDOWN:
			if self.in_overlay:
				self.in_overlay = False
				if self.gallery.click(event.pos):
					self.state.game.switch_to(2)
				elif self.random.click(event.pos):
					self.load_course(self.state.courses[int(random.random()*(len(self.state.courses)-1))])
				else:
					rating = 1
					for star in self.stars:
						if star.mouse_in(event.pos):
							self.in_overlay = True
							if self.course.rate(rating):
								self.course.rating = rating
								if self.course.id != 0:
									self.course.save("download_"+str(self.course.id))
								self.state.game.switch_to(2)
							return True
						rating += 1
				return True
			#print event.pos
			vec = self.course.cue.body.position
			
			if util.distance(vec,vec2d.Vec2d(event.pos)) < self.course.cue.radius+4:
				self.drag_start = vec
		return True